@@ -5,7 +5,7 @@ use warnings;
 package Hash::Ordered;
 # ABSTRACT: A fast, pure-Perl ordered hash class
 
-our $VERSION = '0.009';
+our $VERSION = '0.010';
 
 use Carp ();
 
@@ -179,8 +179,11 @@ sub get {
 sub set {
     my ( $self, $key ) = @_; # don't copy $_[2] in case it's large
     if ( !exists $self->[_DATA]{$key} ) {
+        my $keys = $self->[_KEYS];
+        if ( my $indx = $self->[_INDX] ) {
+            $indx->{$key} = @$keys ? $indx->{ $keys->[-1] } + 1 : 0;
+        }
         CORE::push @{ $self->[_KEYS] }, "$key"; # stringify key
-        $self->[_INDX]{$key} = $#{ $self->[_KEYS] } if $self->[_INDX];
     }
     return $self->[_DATA]{$key} = $_[2];
 }
@@ -290,13 +293,15 @@ sub clear {
 
 sub push {
     my $self = CORE::shift;
-    my ( $data, $keys, $indx ) = @$self;
+    my ( $data, $keys ) = @$self;
     while (@_) {
         my ( $k, $v ) = splice( @_, 0, 2 );
         $self->delete($k) if exists $data->{$k};
         $data->{$k} = $v;
+        if ( my $indx = $self->[_INDX] ) {
+            $indx->{$k} = @$keys ? $indx->{ $keys->[-1] } + 1 : 0;
+        }
         CORE::push @$keys, "$k"; # stringify keys
-        $indx->{$k} = $#$keys if $indx;
     }
     return @$keys - $self->[_GCNT];
 }
@@ -304,8 +309,11 @@ sub push {
 #pod =method pop
 #pod
 #pod     ($key, $value) = $oh->pop;
+#pod     $value = $oh->pop;
 #pod
 #pod Removes and returns the last key-value pair in the ordered hash.
+#pod In scalar context, only the value is returned.  If the hash is empty,
+#pod the returned key and value will be C<undef>.
 #pod
 #pod =cut
 
@@ -317,7 +325,7 @@ sub pop {
     }
     else {
         my $key = CORE::pop @{ $self->[_KEYS] };
-        return $key, delete $self->[_DATA]{$key};
+        return defined($key) ? ( $key, delete $self->[_DATA]{$key} ) : ();
     }
 }
 
@@ -335,13 +343,13 @@ sub pop {
 
 sub unshift {
     my $self = CORE::shift;
-    my ( $data, $keys, $indx ) = @$self;
+    my ( $data, $keys ) = @$self;
     while (@_) {
         my ( $k, $v ) = splice( @_, -2, 2 );
         $self->delete($k) if exists $data->{$k};
         $data->{$k} = $v;
         CORE::unshift @$keys, "$k"; # stringify keys
-        $indx->{$k} = -( ++$self->[_OFFS] ) if $indx;
+        $self->[_INDX]{$k} = -( ++$self->[_OFFS] ) if $self->[_INDX];
     }
     return @$keys - $self->[_GCNT];
 }
@@ -349,8 +357,11 @@ sub unshift {
 #pod =method shift
 #pod
 #pod     ($key, $value) = $oh->shift;
+#pod     $value = $oh->shift;
 #pod
 #pod Removes and returns the first key-value pair in the ordered hash.
+#pod In scalar context, only the value is returned.  If the hash is empty,
+#pod the returned key and value will be C<undef>.
 #pod
 #pod =cut
 
@@ -362,7 +373,7 @@ sub shift {
     }
     else {
         my $key = CORE::shift @{ $self->[_KEYS] };
-        return $key, delete $self->[_DATA]{$key};
+        return defined($key) ? ( $key, delete $self->[_DATA]{$key} ) : ();
     }
 }
 
@@ -657,7 +668,7 @@ Hash::Ordered - A fast, pure-Perl ordered hash class
 
 =head1 VERSION
 
-version 0.009
+version 0.010
 
 =head1 SYNOPSIS
 
@@ -805,8 +816,11 @@ Returns the number of keys after the push is complete.
 =head2 pop
 
     ($key, $value) = $oh->pop;
+    $value = $oh->pop;
 
 Removes and returns the last key-value pair in the ordered hash.
+In scalar context, only the value is returned.  If the hash is empty,
+the returned key and value will be C<undef>.
 
 =head2 unshift
 
@@ -821,8 +835,11 @@ Returns the number of keys after the unshift is complete.
 =head2 shift
 
     ($key, $value) = $oh->shift;
+    $value = $oh->shift;
 
 Removes and returns the first key-value pair in the ordered hash.
+In scalar context, only the value is returned.  If the hash is empty,
+the returned key and value will be C<undef>.
 
 =head2 merge
 
@@ -1168,7 +1185,7 @@ David Golden <dagolden@cpan.org>
 
 =head1 CONTRIBUTORS
 
-=for stopwords Benct Philip Jonsson Mario Roy
+=for stopwords Benct Philip Jonsson David Golden Mario Roy
 
 =over 4
 
@@ -1178,6 +1195,10 @@ Benct Philip Jonsson <bpjonsson@gmail.com>
 
 =item *
 
+David Golden <xdg@xdg.me>
+
+=item *
+
 Mario Roy <marioeroy@gmail.com>
 
 =back
